@@ -19,6 +19,7 @@ from maitai._inference import (
 )
 from maitai._types import (
     Body,
+    EvaluateCallback,
     Headers,
     MaitaiChunk,
     MaitaiCompletion,
@@ -26,21 +27,14 @@ from maitai._types import (
     ToolFunction,
 )
 from maitai._utils import (
+    OpenAISchemaGenerator,
     convert_open_ai_chat_completion_chunk,
     convert_openai_chat_completion,
     get_chat_completion_params,
     required_args,
 )
 from maitai.tools import Tools
-from maitai_common.utils.proto_utils import openai_messages_to_proto
-from maitai_common.utils.types import EvaluateCallback
-from maitai_gen.chat import (
-    ChatCompletionParams,
-    ChatCompletionRequest,
-    ChatMessage,
-    ClientParams,
-    EvaluationContentType,
-)
+from maitai_gen.chat import ChatCompletionParams, ClientParams, EvaluationContentType
 from maitai_gen.config import InferenceLocations
 from maitai_gen.inference import InferenceStreamResponse
 from maitai_gen.metric import RequestTimingMetric
@@ -293,36 +287,38 @@ class AsyncCompletions:
             top_p=top_p,
             user=user,
             parallel_tool_calls=parallel_tool_calls,
+            extra_headers=extra_headers,
         )
         if server_side_inference:
             response_timeout = None
             if isinstance(timeout, float) or isinstance(timeout, int):
                 response_timeout = timeout
-            chat_request: ChatCompletionRequest = ChatCompletionRequest(
-                application_ref_name=application_ref_name,
-                session_id=session_id,
-                reference_id=reference_id,
-                action_type=intent,
-                apply_corrections=apply_corrections,
-                evaluation_enabled=evaluation_enabled,
-                params=completion_params,
-                return_evaluation=True if callback else False,
-                context_retrieval_enabled=context_retrieval_enabled,
-                context_query=context_query,
-                fallback_model=fallback_model,
-                user_id=user_id,
-                assistant=assistant,
-                auth_keys=config.auth_keys,
-                metadata=metadata,
-                safe_mode=safe_mode,
-                extra_headers=extra_headers or {},
-            )
+            request_dict = {
+                "application_ref_name": application_ref_name,
+                "session_id": session_id,
+                "reference_id": reference_id,
+                "action_type": intent,
+                "apply_corrections": apply_corrections,
+                "evaluation_enabled": evaluation_enabled,
+                "params": completion_params,
+                "return_evaluation": True if callback else False,
+                "context_retrieval_enabled": context_retrieval_enabled,
+                "context_query": context_query,
+                "fallback_model": fallback_model,
+                "user_id": user_id,
+                "assistant": assistant,
+                "auth_keys": config.auth_keys.to_dict(),
+                "metadata": metadata,
+                "safe_mode": safe_mode,
+            }
+
             if self._client_params is not None:
-                chat_request.client_params = self._client_params
+                request_dict["client_params"] = self._client_params.to_dict()
+
             try:
                 timing.time_pre_request = time.time()
                 response = await maitai.Inference.infer_async(
-                    chat_request, callback, response_timeout
+                    request_dict, callback, response_timeout
                 )
                 if stream:
                     if "self" in retry_args:
@@ -352,7 +348,9 @@ class AsyncCompletions:
                     timing.request_id = resp.chat_completion_response.request_id
                     timing.time_request_end = time.time()
                     maitai.Inference.store_request_timing_data(timing)
-                    return MaitaiCompletion(resp.chat_completion_response)
+                    return MaitaiCompletion(
+                        resp.chat_completion_response, response_format=response_format
+                    )
                 raise InferenceException("No data received")
             except MaitaiConnectionError as e:
                 if "gpt" not in model and fallback_model and "gpt" in fallback_model:
@@ -368,6 +366,13 @@ class AsyncCompletions:
                     default_query=self._client_params.default_query or None,
                     base_url=self._client_params.base_url or None,
                 )
+            if hasattr(response_format, "model_json_schema"):
+                response_format = {
+                    "type": "json_schema",
+                    "json_schema": response_format.model_json_schema(
+                        schema_generator=OpenAISchemaGenerator, mode="serialization"
+                    ),
+                }
             response = await self._client.chat.completions.create(
                 messages=messages,
                 model=model,
@@ -413,20 +418,19 @@ class AsyncCompletions:
             else:
                 timing.time_request_end = time.time()
                 maitai_completion = convert_openai_chat_completion(response)
-                proto_messages = openai_messages_to_proto(messages)
-                proto_messages.append(
-                    ChatMessage(
-                        role="assistant",
-                        content=maitai_completion.choices[0].message.content,
-                    )
-                )
+                full_conv = messages + [
+                    {
+                        "role": "assistant",
+                        "content": maitai_completion.choices[0].message.content,
+                    }
+                ]
                 if evaluation_enabled:
                     await maitai.Evaluator.evaluate_async(
                         session_id=session_id,
                         reference_id=reference_id,
                         intent=intent,
                         content_type=EvaluationContentType.MESSAGE,
-                        content=proto_messages,
+                        content=full_conv,
                         application_ref_name=application_ref_name,
                         callback=callback,
                         chat_completion_response=maitai_completion,
@@ -447,7 +451,9 @@ class AsyncCompletions:
                         timing=timing,
                         metadata=metadata,
                     )
-                return MaitaiCompletion(maitai_completion)
+                return MaitaiCompletion(
+                    maitai_completion, response_format=response_format
+                )
 
 
 async def _process_async_openai_stream(
@@ -464,7 +470,6 @@ async def _process_async_openai_stream(
     metadata: dict = {},
 ) -> AsyncIterable[MaitaiChunk]:
     full_body = ""
-    proto_messages = openai_messages_to_proto(messages)
     last_chunk = None
     async for chunk in stream:
         if last_chunk is None:
@@ -482,13 +487,13 @@ async def _process_async_openai_stream(
     if last_chunk is None:
         return
     if evaluation_enabled:
-        proto_messages.append(ChatMessage(role="assistant", content=full_body))
+        full_conv = messages + [{"role": "assistant", "content": full_body}]
         await Evaluator.evaluate_async(
             session_id=session_id,
             reference_id=reference_id,
             intent=intent,
             content_type=EvaluationContentType.MESSAGE,
-            content=proto_messages,
+            content=full_conv,
             application_ref_name=application_ref_name,
             callback=callback,
             chat_completion_chunk=last_chunk,
